 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.loext.its.config;

import de.init.its.util.Util;

/**
 * Global Configuration for ITS MLW Extension
 * 
 * TODO: Move all Available Configuration here
 * TODO: Further extension -> externalize all configuration
 * 
 * @author mkandora
 *
 */
public final class GlobalConfiguration {
	
	private GlobalConfiguration() {}
	
	/**
	 * The UNO Protocol Handler URL
	 * 
	 * FIXME: Reconfigure URL for canonical naming
	 */
	public static final String PROTOCOL_HANDLER_URL = "org.openoffice.optionspagedemo";
	
	/**
	 * Number of Columns for Locale Filter Textfield 
	 */
	public static final Integer LOCALE_FILTER_TEXTFIELD_NUM_COLS = 23;
	
	/**
	 * Displays the default delimiter used for bookmarks
	 */
	public static String ITS_DATACATEGORY_MARKER_DELIMITER = "[#]";
	
	/**
	 * The begin marker value for its data category booksmarks, that use its for later post-processing
	 */
	public static String ITS_DATACATEGORY_MARKER 	= "its-" 	+ ITS_DATACATEGORY_MARKER_DELIMITER;
	
	/**
	 * The regular expression pattern to match its bookmark id and marker strings
	 */
	public static String ITS_DATACATEGORY_MARKER_REGEX_PATTERN = "(its)\\-((begin|end)\\-)?(\\[(its\\-\\d*\\]))";
	
	/**
	 * Configuration Service class
	 * 
	 * @author mkandora
	 *
	 */
	public final static class ConfigurationService
	{
		/**
		 * 
		 * 
		 * @param itsDataCategoryId
		 * @return
		 */
		public static String constructMarker(final String itsDataCategoryId) {
			return ITS_DATACATEGORY_MARKER.replaceAll("#", itsDataCategoryId);
		}
		
		/***
		 * Extracts the 
		 * @param marker
		 * @return
		 */
		public static String extractIdFromMarker(final String marker) {
			if(marker.matches(ITS_DATACATEGORY_MARKER_REGEX_PATTERN)) {
				return Util.crop(marker, "[", "]"); 
			}
			return "";
		}
	}
	
}
