 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.loext.its.dialogs.ui;

import java.io.File;
import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import org.junit.experimental.categories.Categories;

import de.init.its.ds.ItsAttributes;
import de.init.its.ds.ItsTagElement;
import de.init.its.ds.ItsAttributes.DataCategories;
import de.init.its.util.Util.Executor;
import de.init.loext.its.dialogs.ui.ItsDialogs.ItsDialogBase;
import de.init.loext.its.dialogs.ui.ItsDialogs.TerminateConfigurationListener;
import de.init.uno.text.MarkupService;

/**
 * Base model for all {@linkplain DataCategories}
 * to store all modified values for easy accessing from the outside.
 * 
 * @author mkandora
 *
 */
public final class ItsDialogModel {
	
	/**
	 * Wrapper class to circumvent the reception of the same event twice
	 * from LibreOffice. THIS IS A HACK AND IS CONSIDERED HARMFUL. 
	 * REMOVE THIS CLASS; IF THERE IS A WAY TO SUPRESS THE MULTIPLE FIRING
	 * OF THE RESPECTIVE LIBREOFFICE EVENT
	 * 
	 * @author mkandora
	 *
	 */
	public static class ConcurrentDialogTask implements Runnable
	{
		private static final ConcurrentDialogTask task = new  ConcurrentDialogTask();
		private static final AtomicBoolean display = new AtomicBoolean(false);
		private Runnable runnable;
		
		private ConcurrentDialogTask() {/* NOP */}
		
		/**
		 * sets a task for later scheduling
		 * @param run
		 * @return
		 */
		public static ConcurrentDialogTask setTask(final Runnable run) {
			task.runnable = run;
			return task;
		}
		
		/**
		 * Starts executing a scheduled task for execution
		 * @return
		 */
		public ConcurrentDialogTask start() {
			new Thread(this).start();
			return this;
		}
		
		/**
		 * resets the internal lock
		 */
		public static synchronized void reset() {
			display.set(false);
		}
		
		@Override
		public void run() {
			if(!display.get()) {
				display.set(true);
				try {
					runnable.run();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				display.set(false);
			}
		}
		
	}
	
	private File lastDir;
	
	private Map<ItsDialogBase, Map<ItsAttributes, List<String>>> 	views;
	private Map<DataCategories, ItsDialogBase> 						datacategories;
	private JFileChooser 											filechooser;
	
	private static ItsDialogModel 									model;
	
	private Map<DataCategories, String> nameMapping;
	
	
	private ItsDialogModel() {
		this.datacategories 		= new HashMap<ItsAttributes.DataCategories, ItsDialogs.ItsDialogBase>();
		this.views 	= new HashMap<ItsDialogBase, Map<ItsAttributes,List<String>>>();
		this.nameMapping = new HashMap<DataCategories, String>();
		
		// initialize name mapping
		nameMapping.put(DataCategories.TRANSLATE, "Translate");
		nameMapping.put(DataCategories.LOCALIZATIONNOTE, "Localization Note");
		nameMapping.put(DataCategories.TERMINOLOGY, "Terminology");
		nameMapping.put(DataCategories.LOCALEFILTER, "Locale Filter");
		
		createFC();
	}

	private void createFC() {
		filechooser = new JFileChooser();
		filechooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
		filechooser.setFileFilter(new FileFilter() {
			
			@Override
			public String getDescription() {
				
				// TODO: localize
				return "(*.xlf, *.xliff) XLIFF Document File";
			}
			
			@Override
			public boolean accept(File f) {
				return f.isDirectory() || f.getName().endsWith("xlf") || f.getName().endsWith("xliff");
			}
		});
	}
	
	/**
	 * displays the file chooser dialog
	 * @param save set to true to show save dialog, open otherwise
	 * @return
	 */
	public void displayFC(final boolean save, final Executor<File> exe) {
		// TODO Auto-generated method stub
		if(lastDir != null) 
			filechooser.setCurrentDirectory(lastDir);
		
				File file = null;
				int option = 0;
				if(save) {
					option = filechooser.showSaveDialog(null);
				} else {
					option = filechooser.showOpenDialog(null);
				}
				if(option == JFileChooser.APPROVE_OPTION) {
					file = filechooser.getSelectedFile();
					exe.execute(file);
				}
				lastDir = file.getParentFile();
	}
	
	/**
	 * Returns a singleton instance of the model
	 * @return
	 */
	public static ItsDialogModel getModel() {
		return model != null ? model : (model = new ItsDialogModel());
	}
	
	/**
	 * Registers a view for this model
	 * 
	 * @param dialog
	 */
	protected void registerView(final ItsDialogBase dialog) {
		this.views.put(dialog, null);
	}
	
	/**
	 * registers a {@linkplain DataCategories} with an {@link ItsDialogBase}
	 * 
	 * @param category
	 * @param dialog
	 */
	protected void registerDc(final DataCategories category, final ItsDialogBase dialog) {
		this.datacategories.put(category, dialog);
	}
	
	/**
	 * Displays the view for the respective {@linkplain ItsAttributes}
	 * 
	 * @param attribute the attribute to show 
	 * @param listener provide the save cancel operation externally
	 */
	public ItsDialogBase load(final DataCategories category, final TerminateConfigurationListener listener) {
		if(!datacategories.containsKey(category)) {
			try {
				for(Class<?> clazz : ItsDialogs.class.getClasses()) {
					ItsDialogView a = clazz.getAnnotation(ItsDialogView.class);
					if(clazz.isAnnotationPresent(ItsDialogView.class) && !datacategories.containsKey(category) && a.category().equals(category)) {
						
						// autoreg ui components
						Constructor<?> constructor = clazz.getConstructor(String.class, TerminateConfigurationListener.class);
						String title = "Edit ".concat(nameMapping.get(a.category()));
						ItsDialogs.ItsDialogBase instance = (ItsDialogs.ItsDialogBase) constructor.newInstance(title, listener);
						registerDc(a.category(), instance);
						registerView(instance);
						break;
					}
				}
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		return datacategories.get(category);
	}
	
	/**
	 * Returns all values that are saved for this model
	 * 
	 * @param dialog
	 * @return
	 */
	public Map<ItsAttributes, List<String>> getValues(final ItsDialogBase dialog) {
		return views.get(dialog);
	}
	
	/**
	 * Sets the values for a dialog
	 * 
	 * @param dialog
	 * @param values
	 */
	public void setValues(final ItsDialogBase dialog, final Map<ItsAttributes, List<String>> values) {
		this.views.put(dialog, values);
	}
	
}
