 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.loext.managers;

import java.io.File;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import com.sun.star.frame.XFrame;

import de.init.its.spi.ItsElementManager;
import de.init.its.util.ItsSerializerService;

/**
 * Service class to handle session for various documents
 * accessible by the user
 * 
 * @author m.kandora
 * 
 */
public class ItsSessionManager {
	
	private static Map<XFrame, ItsElementManager> sessions = new HashMap<XFrame, ItsElementManager>();
	
	private ItsSessionManager() {}
	
	/**
	 * Returns an ItsManager Session for a specific Frame (eg. a new TextEditor Window)
	 * @param _url
	 * @param frame
	 * @return
	 * @throws Exception
	 */
	public static ItsElementManager getSession(final String _url, final XFrame frame) throws Exception {
		String url = null;
		if(_url != null && !_url.isEmpty()) url = _url.endsWith(".its-manager.ser") ? _url : _url.concat(".its-manager.ser");
		if(sessions.containsKey(frame)) {
			return sessions.get(frame);
		} else if(url != null && new File(new URI(url)).exists()) {
			sessions.put(frame, ItsSerializerService.deserialise(url, ItsElementManager.class));
			return sessions.get(frame);
		} else {
			sessions.put(frame, ItsElementManager.getManager(true));
			return sessions.get(frame);
		}
	}
	
	/**
	 * Clears the session for a specific Frame
	 * @param _url
	 * @param frame
	 */
	public static void clearSession(final String _url, final XFrame frame) {
		sessions.remove(frame);
	}
}