 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.service;

import static de.init.util.logging.SimpleLog.log;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.sun.star.frame.XModel;
import com.sun.star.text.XBookmarksSupplier;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

import de.init.its.api.ds.IitsTagElement;
import de.init.its.api.spi.IitsElementManager;
import de.init.its.ds.ItsAttribute;
import de.init.its.ds.ItsAttributes;
import de.init.its.spi.ItsElementManager;

/**
 * Managing class for ITS markup in XTextDocuments
 * 
 * @author mkandora
 *
 */
public class ItsManager {


	private static ItsManager instance; 

	private IitsElementManager manager;
	private XBookmarksSupplier supplier;
	//	private XBookmarkInsertTool inserter;


	/**
	 * returns a singleton instance of the itsmanager
	 * @param doc
	 * @return
	 */
	public final static ItsManager getItsManager(final XTextDocument doc) {
		return instance != null ? instance : (instance = new ItsManager(doc));	
	}

	private ItsManager(final XTextDocument docBase) {
		log("ctor {} loading...", ItsManager.class.getName());
		this.supplier 	= UnoRuntime.queryInterface(XBookmarksSupplier.class, docBase);
		log("BookmarksSupplier succesfully loaded: {}", this.supplier);
		//		this.inserter 	= UnoRuntime.queryInterface(XBookmarkInsertTool.class, docBase);
		//		log("BookmarksInserterTool successfully loaded: {}", this.inserter);
		this.manager 	= ItsElementManager.getManager();
		log("ITSElementManager sucessfully loaded: {}", this.manager );
	}

	/**
	 * Returns a List of all ITS Ids. The LibreOffice Bookmarks mechanism
	 * will be used for this. Internally, the {@linkplain XBookmarksSupplier} will be
	 * used to retrieve the names of all bookmarks
	 * 
	 * @return a list of all its tag ids
	 */
	public List<String> getItsTagIds() {
		String[] enames = this.supplier.getBookmarks().getElementNames();

		// FIXME: is this enough?
		return Arrays.asList(enames);
	}

	/**
	 * Inserts an its tag into the {@link XTextDocument}
	 * @param xContext 
	 * 
	 * @param range the {@linkplain XTextRange}
	 * @param dataCategoryName 
	 * @param currentSelection 
	 * @param attrib the ITS Tag Attribute Name {@linkplain ItsAttributes}
	 * @param values a set of values for the attribute
	 * @return the {@link XTextContent} with bookmark set
	 */
	public String insertDataCategory(XComponentContext xContext, final XModel model, final XTextRange range, final Map<String, String> map, 
			String dataCategoryName) {

		// TODO: check hierarchy.. how?
		// TODO: insert all attributes from 
		IitsTagElement element = null; //manager.insert(attrib, values);

		// UGLY HACK: save the first, add all successive attributes to the inserted element
		// see todos above
		int index  = 0;
		for(String dcname : map.keySet()) {
			String dcvalue = map.get(dcname);
			if(index == 0) {
				element = manager.insert(dcname, dcvalue);
				// TODO: check for errors
				if(manager.hasErrors()) {
					// TODO: what to do when errors have occured?
					// launch a dialog containing the error message
					log(" Bookmark insertion NOT successful");
					log(" manager.hasErrors() {} ", manager.getErrors() );
					manager.clearErrors();
					return "error";
				}
				index++;
			} else {
				element.addAttribute(new ItsAttribute(dcname, dcvalue));
			}
			// TODO: save first element
			// insert remaining configurations into existing ItsTagElement
			// FIXME: move this behaviour to itstagelementmanager
		}
		log("successfully inserted Its DataCategory {}", element);

		return element.getId().getString();
	}

}
