 /* ===========================================================================
 *   Copyright (C) 2013 by ]Init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.service;

import static de.init.util.logging.SimpleLog.log;

import java.net.URI;

import com.sun.star.beans.PropertyValue;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XStorable;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.UnoRuntime;

import de.init.its.process.re.ItsProcessor;
import de.init.its.spi.ItsElementManager;
import de.init.its.util.Util;
/**
 * <p>
 * Main Class for ODT File import and export.
 * </p>
 * <p>
 * Use this class to perform any processing concerning ITS markup in ODT files. In General
 * this class SHALL be used to load and save TextDocument, that contain ITS markup. OpenOffice
 * forbids the direct integration into the io processing, hence this class shall be
 * called from a controller linked to some UI element: A toolbar or something. 
 * </p>
 * 
 * @author mkandora
 *
 */
public class ItsOdtIOManager {
	
	private static ItsOdtIOManager ref;
	private XComponentLoader loader;
	
	/**
	 * Create a new ItsOdtIoManager for writing and loading existing ODT Files
	 * 
	 * <p>
	 * <b>see</b> {@link UnoRuntime#queryInterface(XComponentLoader.class, XDesktop)}
	 * </p
	 * 
	 * @param loader the {@link XComponentLoader} retrieved from com.sun.star.text.Desktop and its 
	 * service interface {@link XDesktop}
	 * >
	 */
	private ItsOdtIOManager(final XComponentLoader loader) {
		this.loader		= loader;
		
	}
	
	/**
	 * Obtains an instance of the {@link ItsOdtIOManager}
	 * 
	 * @param loader the {@link XComponentLoader} derived from {@link XDesktop} 
	 * @return an instance of the {@link ItsOdtIOManager}
	 */
	public static ItsOdtIOManager getIoManager(final XComponentLoader loader) {
		return ref != null ? ref : (ref = new ItsOdtIOManager(loader));
	}
	/**
	 * Imports an ODT Writer document
	 * 
	 * @param resource
	 * @return
	 * @throws Exception
	 */
	public XTextDocument importDocument(final String resource) throws Exception {
		// parse document first to obtain all possibly available ITS markup Information
		// retrievable via ItsElementManager
		String target = Util.format("file://{}", resource);
		XTextDocument document = (XTextDocument) loader.loadComponentFromURL(target, "_default", 0, new PropertyValue[]{});
		log("successfully loaded xtextdocument? {}", document);
		return document;
	}
	
	
	/**
	 * Exports (Saves) a document. 
	 * 
	 * <p>
	 * 	In order to circumvent a direct ODT File Implementation, the ODT File will be saved natively via
	 * the UNO API, the pre-saved document will be further processed by {@link ItsProcessor} to insert the 
	 * appropriate data categories. 
	 * </p>
	 * 
	 * @param xDocument the {@link XTextDocument} to save
	 * @param doctype
	 * @return
	 * @throws Exception 
	 */
	public boolean exportDocument(final URI resource, final XTextDocument xDocument /*, final ItsODTDocumentType doctype , XMultiServiceFactory xMSF*/) throws Exception {
		try {
			XStorable xstoreable = UnoRuntime.queryInterface(XStorable.class, xDocument);
			String target = "file://" + resource.getPath(); //Util.format("file:///{}", resource);
			
			// fix
			if(!target.endsWith("odt")) target += ".odt";
			
			// save document as odt
			xstoreable.storeAsURL(target, new PropertyValue[]{});
			
			// ?
			ItsElementManager manager = ItsElementManager.getManager();
			
			// fix this or delete ItsOdtManager class
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
}