 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.uno.text;

import static de.init.util.logging.SimpleLog.log;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.openoffice.demo.Range;

import com.sun.star.beans.PropertyVetoException;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XNameAccess;
import com.sun.star.container.XNamed;
import com.sun.star.frame.XController;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XModel;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.text.XBookmarksSupplier;
import com.sun.star.text.XText;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.view.XSelectionSupplier;

import de.init.its.api.ds.IitsTagElement;
import de.init.its.api.process.OdtStyleConfiguration;
import de.init.its.ds.ItsTagElement;
import de.init.its.ds.ItsAttributes.DataCategories;
import de.init.its.spi.ItsElementManager;
import de.init.loext.its.config.GlobalConfiguration;
import de.init.loext.its.config.GlobalConfiguration.ConfigurationService;
import de.init.loext.managers.ItsSessionManager;
import de.init.service.ItsManager;

/**
 * Service class to manage bookmarks 
 * 
 * @author m.kandora
 *
 */
public class MarkupService 
{
	private final static MarkupService service = new MarkupService();
	
	private IitsTagElement emptyElement;
	
	private MarkupService() {
		emptyElement = new ItsTagElement();
	}
	
	/**
	 * Returns a singleton instance of the MarkupService
	 * @return
	 */
	public static MarkupService getMarkupService() {
		return service;
	}
	
	/**
	 * Inserts a Bookmark
	 * 
	 * @param comp
	 * @param xframe
	 * @param update
	 */
	public void insertBookmark(
			final XComponentContext xContext, 
			final XFrame xframe, 
			final boolean update, 
			final DataCategories datacategory,
			final IitsTagElement element) 
	throws Exception
	{
		
		XModel xDocumentModel 			= xframe.getController().getModel();
		XController xController 		= xframe.getController();
		XSelectionSupplier xSelSupplier = UnoRuntime.queryInterface(XSelectionSupplier.class, xController );
		Object oSelection 				= xSelSupplier.getSelection();
		XServiceInfo xServInfo 			= UnoRuntime.queryInterface( XServiceInfo.class, oSelection );
		
		if (xServInfo.supportsService("com.sun.star.text.TextRanges") ) 
		{
			XIndexAccess xIndexAccess 	= UnoRuntime.queryInterface( XIndexAccess.class, oSelection);
			
			// handle multiple selections via count
			int count 					= xIndexAccess.getCount();
			XTextRange xTextRange 		= null;
			for ( int i = 0; i < count; i++ ) {
				
				Object selectionByIndex 	= xIndexAccess.getByIndex(i);
				xTextRange 					= UnoRuntime.queryInterface( XTextRange.class, selectionByIndex);
				XTextDocument xTextDocument = UnoRuntime.queryInterface( XTextDocument.class, xDocumentModel);
				
				if(update) continue;
					
				//////// ---------------------------------------
				XTextDocument xtextDocument = UnoRuntime.queryInterface(XTextDocument.class, xDocumentModel);
				XTextCursor cursor 			= UnoRuntime.queryInterface(XTextCursor.class, xtextDocument.getText().getStart());
				cursor.gotoRange(xTextRange, false); // set cursor to begin of selection
				
				String bookMarker = GlobalConfiguration.ConfigurationService.constructMarker(element.getId().getString());
				
				XMultiServiceFactory xFactory 	= UnoRuntime.queryInterface(XMultiServiceFactory.class, xTextDocument);
				Object bookmark;
				try {
					bookmark = xFactory.createInstance("com.sun.star.text.Bookmark");
					XText xTextAll = xTextDocument.getText(); //  cursor.getText(); // 

					// set the name from the bookmark
					com.sun.star.container.XNamed xNameAccess = null;
					xNameAccess = UnoRuntime.queryInterface( XNamed.class, bookmark);

					xNameAccess.setName(bookMarker);
					XTextContent xTextContentOnBookmark =UnoRuntime.queryInterface( XTextContent.class, xNameAccess);
					xTextAll.insertTextContent(cursor, xTextContentOnBookmark, Boolean.TRUE /* replace range from cursor */);
					
					// visualize
					highlight(cursor, datacategory);
					
					// return to default style
					cursor.gotoEnd(false);
					defaultstyle(cursor);
				} catch (com.sun.star.uno.Exception e) {
				}
				// prepare cursor for correct range of visualizeSelection
				cursor.gotoRange(xTextRange, false);
			}
		}
	}
	
	/**
	 * Removes the boomark from uno context
	 * 
	 * @param xContext
	 * @param frame
	 * @param id
	 */
	public void removeBookmark(final XComponentContext xContext, final XFrame frame,  final String id)  {
		try {
			XTextDocument xtextDocument 	= UnoRuntime.queryInterface(XTextDocument.class, frame.getController().getModel());
			XBookmarksSupplier xBookmarksSupplier = UnoRuntime.queryInterface(XBookmarksSupplier.class, xtextDocument);
			XNameAccess xNamedBookmarks = xBookmarksSupplier.getBookmarks();
			UnoRuntime.queryInterface(
					XTextContent.class, 
					xNamedBookmarks.getByName(ConfigurationService.constructMarker(id))).dispose();
			
			XTextCursor cursor = UnoRuntime.queryInterface(XTextCursor.class, xtextDocument.getText().getStart());
			cursor.gotoRange(fetchSelection(xContext, frame), false);
			defaultstyle(cursor);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Creates a {@link Range} Object from a selection
	 * 
	 * @param selection the current selection
	 * @param cursor the text cursor obtained by an {@link XTextDocument} 
	 * @return a {@link Range}
	 */
	private Range createRange(XTextRange selection, XTextCursor cursor) {
		cursor.gotoRange(selection.getStart(), false);
		int start						= getSelectionBegin(cursor);
		cursor.gotoRange(selection.getEnd(), false);
		return new Range(start, getSelectionBegin(cursor));
	}
	
	/**
	 * Return cursor location of the bookmark specified by nameBookmark.
	 * Throws IllegalArgumentException if the nameBookmark is not known.
	 * 
	 * @param xComponent text component where the bookmark is retrieved from
	 * @param name name of the bookmark.
	 * @return cursor location
	 */
	public static XTextCursor createCursorByBookmark(XComponent xComponent, String name) {
		XBookmarksSupplier xBookmarksSupplier = UnoRuntime.queryInterface(XBookmarksSupplier.class, xComponent);
		XNameAccess xNamedBookmarks = xBookmarksSupplier.getBookmarks();
		
		try {
			Object bookmark 			= xNamedBookmarks.getByName(name);
			XTextContent xTextContent 	= UnoRuntime.queryInterface(XTextContent.class, bookmark);
			XTextRange xTextRange 		= xTextContent.getAnchor();
			XText xLocalText 			= xTextRange.getText();
			XTextCursor xLocalCursor 	= xLocalText.createTextCursorByRange(xTextRange);
			
			return xLocalCursor;
		} catch (NoSuchElementException e) {
			log("bookmark '" + name + "' does not exist");
		} catch (WrappedTargetException e) {
			log("unknown problem with bookmark '" + name + "'");
		}
		return null;
	}

	
	/**
	 * Highlights a selected piece of text according to the stylemapping
	 * for {@linkplain Datacategories}
	 * 
	 * @param xTextCursor
	 * @param datacategory
	 * @throws UnknownPropertyException
	 * @throws PropertyVetoException
	 * @throws IllegalArgumentException
	 * @throws WrappedTargetException
	 */
	private void highlight(final XTextCursor xTextCursor, final DataCategories datacategory)
			throws UnknownPropertyException, PropertyVetoException,
			IllegalArgumentException, WrappedTargetException 
	{
		// see http://www.openoffice.org/api/docs/common/ref/com/sun/star/style/CharacterProperties.html
		XPropertySet prop = UnoRuntime.queryInterface(XPropertySet.class, xTextCursor);
		
		try {
		Map<String, String> styles = 
				OdtStyleConfiguration.getUnoStyleMappingForDataCategory(datacategory);
		for(String styleName : styles.keySet()) 
		{
			Object style = convertToInt(styles.get(styleName));
			if("solid".equals(style)) style = com.sun.star.awt.FontUnderline.BOLD;
			if("font-color".equals(style)) style = new Integer(0xFF00FF);
			prop.setPropertyValue(styleName, style);
		}
		} catch(Exception e) {
		}
	}
	
	private void defaultstyle(final XTextCursor xTextCursor)
			throws UnknownPropertyException, PropertyVetoException,
			IllegalArgumentException, WrappedTargetException 
	{
		// see http://www.openoffice.org/api/docs/common/ref/com/sun/star/style/CharacterProperties.html
		XPropertySet prop = UnoRuntime.queryInterface(XPropertySet.class, xTextCursor);
		
		try {
			Integer white = new Integer(0xFFFFFF);
			prop.setPropertyValue("CharBackColor", white);
			prop.setPropertyValue("CharColor", new Integer(0));
			prop.setPropertyValue("CharUnderline", com.sun.star.awt.FontUnderline.NONE);
			prop.setPropertyValue("CharOverline", com.sun.star.awt.FontUnderline.NONE);
			prop.setPropertyValue("CharOverlineColor", white);
			prop.setPropertyValue("CharUnderlineColor", white);
			
			
		}
		catch(Exception e) {
		}
	}
	
	/**
	 * converts a hexadecimal string 
	 * @param in
	 * @return
	 */
	private static Object convertToInt(final String in) {
		if(in.matches("\\#[0-9a-fA-F]{6}")) {
			return Integer.parseInt(in.substring(in.indexOf("#") + 1), 16);
		} return in;
	}

	/**
	 * fetch first selection, 
	 * 
	 * 
	 * @param context
	 * @param frame
	 * @return
	 * @throws IndexOutOfBoundsException
	 * @throws WrappedTargetException
	 */
	public static XTextRange fetchSelection(XComponentContext context, XFrame frame) 
	throws IndexOutOfBoundsException, WrappedTargetException 
	{
		XController xController 			= frame.getController();
		XSelectionSupplier xSelSupplier 	= UnoRuntime.queryInterface(XSelectionSupplier.class, xController );
		Object oSelection 					= xSelSupplier.getSelection();
		XServiceInfo xServInfo 				= UnoRuntime.queryInterface(XServiceInfo.class, oSelection );
		if ( xServInfo.supportsService("com.sun.star.text.TextRanges") ) 
		{
			XIndexAccess xIndexAccess 		=	UnoRuntime.queryInterface(XIndexAccess.class, oSelection);
			// handle multiple selections via count
			XTextRange xTextRange 			= null;
			Object selectionByIndex 		= xIndexAccess.getByIndex(0);
			xTextRange 						= UnoRuntime.queryInterface(XTextRange.class, selectionByIndex);
			return xTextRange;
		}
		return null;
	}
	
	/**
	 * returns the innermost bookmark from a selection as an {@link ItsTagElement} or
	 * an <b>empty</b> {@link ItsTagElement}, if no bookmark can be found 
	 * 
	 * @param context the application context
	 * @param frame the frame
	 * @return bookmark as {@link IitsTagElement} (wether empty or not)
	 */
	public IitsTagElement findBookmark(final XComponentContext context, final XFrame frame) {
		
		try {
			// get selection
			XTextRange selection 			= null;
			
			if((selection = fetchSelection(context, frame)) == null) {
				return emptyElement;
			}
			
			XTextDocument xtextDocument 	= UnoRuntime.queryInterface(XTextDocument.class, frame.getController().getModel());
			XTextCursor cursor 				= UnoRuntime.queryInterface(XTextCursor.class, xtextDocument.getText().getStart());
			
			Range selRng = createRange(selection, cursor);
			
			// get all bookmark names from this textdocument
			String[] names 		= getBookmarkNames(xtextDocument);
			
			List<Range> indices = new ArrayList<Range>(names.length);
			for (String bookmark : names) 
			{
				int bookmarkStart = getSelectionBegin(createCursorByBookmark(xtextDocument, bookmark));
				// calculate end for begin-marks only
				int bookmarkEnd = bookmarkStart;
				
				// just check, of it is a valid bookmark
				if (bookmark.matches(GlobalConfiguration.ITS_DATACATEGORY_MARKER_REGEX_PATTERN)) 
				{
					XTextCursor cursorBookmark = createCursorByBookmark(xtextDocument, bookmark);
					
					if (cursorBookmark != null) {
						cursor.gotoRange(cursorBookmark.getStart(), false);
						cursorBookmark.gotoRange(cursorBookmark.getEnd(), false);
						bookmarkEnd = getSelectionBegin(cursorBookmark); 
					}
					// add to valid bookmarks
					indices.add( new Range(bookmark, bookmarkStart, bookmarkEnd) );
				}
			}
			
			Range matchingBookmark = null;
			for (Range range : indices) { 
				boolean match = selRng.match(range);
				if (match) 
				{ 
					matchingBookmark = range;
					break;
				}
			}
			if (matchingBookmark == null) {
				return emptyElement;
			} 
			
			IitsTagElement element = ItsSessionManager.getSession(null, frame).findById(
					ConfigurationService.extractIdFromMarker(matchingBookmark.getName()));
			return element;
		
		} catch(final Exception e) { e.printStackTrace(); }
		
		// return empty element, if no appropriate match can be found
		return emptyElement;
	}
	
	/**
	 * Returns all available bookmarks in the provided document
	 * 
	 * @param document
	 * @return an array of all bookmark names
	 */
	public String[] getBookmarkNames(final XTextDocument document) {
		XBookmarksSupplier service = UnoRuntime.queryInterface(XBookmarksSupplier.class, document);
		XNameAccess bookmarks = service.getBookmarks();
		return bookmarks.getElementNames();
	}
	
	/**
	 * 
	 * @param xtc
	 * @return
	 */
	public int getSelectionBegin(final XTextCursor xtc) {
		int count = 0;
		while(xtc.goLeft((short)1, false)) ++count;
		xtc.goRight((short)count, false);
		return ( count );
	}
	
}
