 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.util.logging;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Simplistic Logging Utility
 * 
 * @author mkandora
 *
 */
public class SimpleLog {
	
	public static String file = "c:/lo_log.txt"; // "c:/temp/lo_log.txt"; // "c:/devel_Okapi/lo_log.txt";
	private static boolean enabled = true;
	
	
	
	/**
	 * logs a message to file
	 * 
	 * @param msg the message
	 * @param depth defines the stacktrace depth to retrieve the actual calling method and class
	 */
	private static void log(String msg, int depth) {
		try {
			File logfile = new File(file);
			if(!logfile.exists()) logfile.createNewFile();
			FileWriter fw = new FileWriter(logfile, true);
			StackTraceElement[] trace = Thread.currentThread().getStackTrace();
			String callerr 	= trace[depth].getClassName();
			String method 	= trace[depth].getMethodName();
			Date time = Calendar.getInstance().getTime();
			String out = "[" + callerr+"#"+method+"()]: " + msg + "\n";
//			fw.append(out);
			fw.append(msg + "\n\n");
			
			fw.flush();
			fw.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * log an exception
	 * 
	 * @param message an additional message
	 * @param exception the exception to log
 	 */
	public static void error(final String message, final Object exception) {
		if(enabled) log("An Exception has occured. msg: {}, exception: {}", message, exception);
	}
	
	/**
	 * logs a stacktrace with a given message
	 *  
	 * @param message an additional message
	 * @param stacktrace the complete stacktrace
	 */
	public static void error(final String message, final Object[] stacktrace) {
		StringBuffer buffer = new StringBuffer();
		String delim = System.getProperty("line.delimiter");
		if(delim == null || delim.isEmpty()) delim = "\n";
		for (int i = 0; i < stacktrace.length; i++) {
			buffer.append(stacktrace[i]);
			buffer.append(delim);
		}
		error(message, buffer.toString());
	}
	
	/**
	 * Simple log an exception
	 * @param error
	 */
	public static void error(Object error) {
		error("", error);
	}
	
	/**
	 * Simply logs a stackstrace
	 * @param stackTrace
	 */
	public static void error(Object[] stackTrace) {
		error("", stackTrace);
	}
	
	/**
	 * Simply log a message
	 * @param msg
	 */
	public static void log(String msg) {
		if(enabled) log(msg, 3);
	}
	
	/**
	 * re-enables this logger. 
	 */
	public static void logEnable() {
		log("Logger has been disabled!", 3);
		enabled = true;
	}
	
	/**
	 * disables the logger. this sets a state flag to false for operation.
	 * use the log file to indicate where the logger has been
	 * disable in order to {@link #logEnable()} it again
	 *
	 */
	public static void logDisable() {
		log("Logger has been enabled!", 3);
		enabled = false;
	}
	
	/**
	 * logs a message with an arbitrary number of arguments
	 * the variable is '{}' without quotes. 
	 * 
	 * @param msg
	 * @param params
	 */
	public static void log(String msg, Object...params) {
		String variable = "\\{\\}";
		StringBuffer sb = new StringBuffer();
		int index = 0;
		Matcher m = Pattern.compile(variable).matcher(msg);
		while(m.find()) {
			Object rplc = ""; 
			if(index < params.length)
				rplc = params[index];
			
			m.appendReplacement(sb, "" + rplc);
			index++;
		}
		m.appendTail(sb);
		log(sb.toString(), 3);
	}

}
