 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/

package org.openoffice.demo;

import static de.init.util.logging.SimpleLog.log;

import com.sun.star.beans.PropertyState;
import com.sun.star.beans.PropertyValue;
import com.sun.star.container.XNameAccess;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 * Provides generic factory methods around configuration access.
 * 
 * @author m.kandora
 */
public class ConfigurationAccess
{
  /**
   * Creates a XNameAccess instance for read and write access to the
   * configuration at the given node path.
   * 
   * @param context
   * @param path
   * @return XNameAccess
   */
  public static XNameAccess createUpdateAccess(XComponentContext context, String path)
  {
    XNameAccess access;
    XMultiServiceFactory xConfig;
    try
    {
      xConfig = (XMultiServiceFactory) UnoRuntime.queryInterface(
        XMultiServiceFactory.class,
        context.getServiceManager().createInstanceWithContext(
          "com.sun.star.configuration.ConfigurationProvider", context));
    }
    catch (com.sun.star.uno.Exception e)
    {
    	log("exception " + e);
      return null;
    }
    Object[] args = new Object[1];
    args[0] = new PropertyValue("nodepath", 0, path,PropertyState.DIRECT_VALUE);
    try
    {
      access = (XNameAccess) UnoRuntime.queryInterface(
        XNameAccess.class, xConfig.createInstanceWithArguments(
          "com.sun.star.configuration.ConfigurationUpdateAccess", args));
    }
    catch (com.sun.star.uno.Exception e)
    {
    	log("exception " + e);
      return null;
    }

    return access;
  }
}
