 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package org.openoffice.demo;


import static de.init.util.logging.SimpleLog.log;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import net.sf.okapi.common.LocaleId;

import com.sun.star.beans.PropertyValue;
import com.sun.star.frame.ControlEvent;
import com.sun.star.frame.DispatchDescriptor;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XDispatch;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XStatusListener;
import com.sun.star.lang.XSingleComponentFactory;
import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.URL;

import de.init.its.api.ds.IitsTagElement;
import de.init.its.ds.ItsAttributes;
import de.init.its.ds.ItsAttributes.DataCategories;
import de.init.its.process.IOResult;
import de.init.its.process.re.ItsProcessor;
import de.init.its.process.re.XLIFFProcessor;
import de.init.its.process.support.Element;
import de.init.its.process.support.ItsTree;
import de.init.its.spi.ItsElementManager;
import de.init.its.util.ItsSerializerService;
import de.init.its.util.Util;
import de.init.its.util.Util.Executor;
import de.init.its.util.Util.MatcherOp;
import de.init.loext.its.dialogs.ui.ItsDialogModel;
import de.init.loext.its.dialogs.ui.ItsDialogModel.ConcurrentDialogTask;
import de.init.loext.its.dialogs.ui.ItsDialogs.ItsDialogBase;
import de.init.loext.its.dialogs.ui.ItsDialogs.TerminateConfigurationListener;
import de.init.loext.managers.ItsSessionManager;
import de.init.service.ItsOdtIOManager;
import de.init.service.UnoServiceManager;
import de.init.uno.text.MarkupService;

/**
 * Basic entry class for all event based functions regarding the ITS-tags
 * 
 * this class is instantiated from UNO per button, box, ...
 * each toggleDropdownButton has some clickable lines
 * each line has one buttonMessage
 * the buttonMessage contains the variable text of the ITS-tag-fields
 * data structure  
 * one buttonMessage per clickable content-line
 * aURL.Path LocalizationNote reflect to one button with some messages
 * some messages per url. path-> url-related objects ?
 *  
 * idea
 * structure for dispatch
 * hashmap key=Url.Path+value, val=buttonMessage 
 *  
 *  
 * dispatch() is called 
 * Url.Path TranslateNote
 * valueFromButtonClick selector: default
 * 
 * 
 * SuggestedNames: ItsExtensionModule, IloEntryPoint, IloMain 
 * 
 * @author mkandora, tmeinert
 *
 */
public class IloMain extends WeakBase implements
com.sun.star.lang.XServiceInfo, com.sun.star.frame.XDispatchProvider,
com.sun.star.lang.XInitialization, com.sun.star.frame.XDispatch,
com.sun.star.frame.XControlNotificationListener
{

	/**
	 * This String is important for any event url calls to this class
	 * The value must be reflected in each configuration file:
	 * Addons.xcu, ProtocolHandler.xcu and others
	 */
	final static String PROTOCOL = "org.openoffice.demo.ilomain";

	// ----
	// Defining all external function call names referenced in addons.xcu
	// ----
	public final static String URL_LOCALIZATION 	= "LocalizationNote";
	public final static String URL_TERMINOLOGY		= "Terminology";
	public final static String URL_LOCALEFILTER 	= "Localefilter";
	public final static String URL_TRANSLATE 		= "Translate";
	public final static String URL_EXPORT			= "Export";
	public final static String URL_IMPORT			= "Import";

	private static final String   implementationName = IloMain.class.getName();
	private static final String[] serviceNames = {"com.sun.star.frame.ProtocolHandler" };

	private XComponentContext context;
	private XFrame frame;
	private XStatusListener xStatusListener;
	private boolean lastButtonState = false; // ist button optisch grau unterlegt

	
	// part of a very ugly hack to circumvent the display of two file choosers
	private long lastTime = 0;
	
	/** this class is instantiated per button, box, ... all lists must be static
	* list of listener per button
	*/
	private static List<XStatusListener> statusListeners = new ArrayList<XStatusListener>();


//	private String url;
	
	// hashmap key=Url.Path+value, val=buttonMessage
//	private static TableOfItsTagFields tableOfItsTagFields = new TableOfItsTagFields();

	public IloMain(XComponentContext xContext) {
		this.context = xContext;
		UnoServiceManager.getServiceManager(context);
		
		try {
			Object desktop =  xContext.getServiceManager().createInstanceWithContext("com.sun.star.frame.Desktop", context);
			this.frame = UnoRuntime.queryInterface(XFrame.class, desktop);
			
//			XTextDocument text = UnoRuntime.queryInterface(XTextDocument.class, this.frame.getController().getModel());

		} catch (Exception e) {
		}
	}


	/**
	 * Is called by CentralRegistrationClass.
	 * @param sImplementationName UNO Service
	 * @return
	 */
	public static XSingleComponentFactory __getComponentFactory( String sImplementationName )
	{
		log("get SingleComponent Factory", sImplementationName);
		XSingleComponentFactory xFactory = null;

		if ( sImplementationName.equals( implementationName ) )
			xFactory = Factory.createComponentFactory(IloMain.class, serviceNames);
		return xFactory;
	}

	/**
	 * Is called by CentralRegistrationClass.
	 * @param registryKey
	 * @return
	 */
	public static boolean __writeRegistryServiceInfo( XRegistryKey registryKey )
	{
		log("Registering Component with regKey {}", registryKey);
		return Factory.writeRegistryServiceInfo(implementationName,serviceNames,registryKey);
	}

	/**
	 * called on start per button
	 * here each button content is initialized
	 * and
	 * Messages are created for further use 
	 * in EditDialog
	 * and TextHandling
	 * 
	 * 
	 */
	@Override
	public void addStatusListener(XStatusListener listener, URL url) {
		xStatusListener = listener;
//		statusListeners.add(listener);
	}

	@Override
	public void dispatch(URL aURL, PropertyValue[] propertyValues) {
		try {
			
			final MarkupService service = MarkupService.getMarkupService();
			final ItsDialogModel model = ItsDialogModel.getModel();
			
			// TODO bind to session
			final ItsElementManager its = ItsSessionManager.getSession(frame.getController().getModel().getURL(), this.frame);//ItsElementManager.getManager();
			
			if(aURL.Protocol.compareTo("org.openoffice.demo.ilomain:") == 0) {
				
				
				// find existing bookmark or retrieb
				final IitsTagElement element = service.findBookmark(context, frame);
				
				if ( aURL.Path.compareTo(URL_LOCALEFILTER) == 0 ) {
					
					ItsDialogBase dialog = model.load(DataCategories.LOCALEFILTER, new TerminateConfigurationListener() {
						
						@Override
						public void doSave(ItsDialogBase dialog) {
							try {
								if(element.isEmpty()) {
									Map<ItsAttributes, List<String>> values = model.getValues(dialog);
									
									if(values.containsKey(ItsAttributes.localeFilterList)) {
										String list = values.get(ItsAttributes.localeFilterList).get(0);
										
										if(!ItsAttributes.localeFilterList.validate(list)) {
											JOptionPane.showMessageDialog(null, 
													"Error! Illegal Locale Filter List Value! Check your entries!", 
													"Error! Illegal Locale Filter List values", 
													JOptionPane.ERROR_MESSAGE);
											return;
										}
									}
									service.insertBookmark(context, frame, false, DataCategories.LOCALEFILTER, its.insert(values));
								} else {
									element.update(model.getValues(dialog));
								}
								dialog.reset();
							} catch (Exception e) {
								e.printStackTrace();
							} 
							dialog.setVisible(false);
						}
						
						@Override
						public void doRemove(ItsDialogBase dialog) {
							IitsTagElement element = service.findBookmark(context, frame);
							its.remove(element.getId().getString());
							service.removeBookmark(context, frame, element.getId().getString());
							dialog.setVisible(false);
						}
						
						@Override
						public void doCancel(ItsDialogBase dialog) {
							// TODO Auto-generated method stub
							dialog.setVisible(false);
						}
					});
					model.setValues(dialog, element.renderAttributes());
					dialog.update().display();
				}
				else if ( aURL.Path.compareTo(URL_TRANSLATE) == 0 ) {
					
					ItsDialogBase dialog = model.load(DataCategories.TRANSLATE, new TerminateConfigurationListener() {
						
						@Override
						public void doSave(ItsDialogBase dialog) {
							// TODO Auto-generated method stub
							try {
								IitsTagElement element = service.findBookmark(context, frame);
								if(element.isEmpty()) {
									service.insertBookmark(context, frame, false, DataCategories.TRANSLATE, its.insert(model.getValues(dialog)));
								} else {
									element.update(model.getValues(dialog));
								}
								dialog.reset();
							} catch (Exception e) {
								e.printStackTrace();
							} 
							dialog.setVisible(false);
						}
						
						@Override
						public void doRemove(ItsDialogBase dialog) {
							IitsTagElement element = service.findBookmark(context, frame);
							its.remove(element.getId().getString());
							service.removeBookmark(context, frame, element.getId().getString());
							dialog.setVisible(false);
						}
						
						@Override
						public void doCancel(ItsDialogBase dialog) {
							dialog.setVisible(false);
							dialog.reset();
						}
					});
					model.setValues(dialog, element.renderAttributes());
					dialog.update().display();
				}
				else if ( aURL.Path.compareTo(URL_LOCALIZATION) == 0 ) {
					ItsDialogBase dialog = model.load(DataCategories.LOCALIZATIONNOTE, new TerminateConfigurationListener() {
						
						@Override
						public void doSave(ItsDialogBase dialog) {
							try {
								IitsTagElement element = service.findBookmark(context, frame);
								if(element.isEmpty()) {
									service.insertBookmark(context, frame, false, DataCategories.LOCALIZATIONNOTE, its.insert(model.getValues(dialog)));
								} else {
									element.update(model.getValues(dialog));
								}
								dialog.reset();
							} catch (Exception e) {
								e.printStackTrace();
							} 
							dialog.setVisible(false);
						}
						
						@Override
						public void doRemove(ItsDialogBase dialog) {
							IitsTagElement element = service.findBookmark(context, frame);
							its.remove(element.getId().getString());
							service.removeBookmark(context, frame, element.getId().getString());
							dialog.setVisible(false);
						}
						
						@Override
						public void doCancel(ItsDialogBase dialog) {
							dialog.setVisible(false);
						}
					});
					model.setValues(dialog, element.renderAttributes());
					dialog.update().display();
					
				}
				
				else if ( aURL.Path.compareTo(URL_TERMINOLOGY) == 0) {
					
					ItsDialogBase dialog = model.load(DataCategories.TERMINOLOGY, new TerminateConfigurationListener() {
						
						@Override
						public void doSave(ItsDialogBase dialog) {
							// TODO Auto-generated method stub
							try {
								IitsTagElement element = service.findBookmark(context, frame);
								if(element.isEmpty()) {
									
									// MK : changed String to URI according to
									// http://www.w3.org/TR/2013/WD-its20-20130820/#terminology
									String annotRef = "terminology|http://www.init.de/en/LibreOfficeWriter";
									
									Map<ItsAttributes, List<String>> attributes = model.getValues(dialog);
									if(attributes.containsKey(ItsAttributes.termConfidence)) {
										attributes.put(ItsAttributes.annotatorsRef, Arrays.asList(annotRef));
									}
									service.insertBookmark(context, frame, false, DataCategories.TERMINOLOGY, its.insert(attributes));
								} else {
									element.update(model.getValues(dialog));
								}
								dialog.reset();
							} catch (Exception e) {
								e.printStackTrace();
							} 
							dialog.setVisible(false);
						}
						
						@Override
						public void doRemove(ItsDialogBase dialog) {
							IitsTagElement element = service.findBookmark(context, frame);
							its.remove(element.getId().getString());
							service.removeBookmark(context, frame, element.getId().getString());
							dialog.setVisible(false);
						}
						
						@Override
						public void doCancel(ItsDialogBase dialog) {
							dialog.setVisible(false);
						}
					});
					model.setValues(dialog, element.renderAttributes());
					dialog.update().display();
				}
				
				if (aURL.Path.compareTo(URL_EXPORT) == 0) {
					
					// suppress the dual display of the save dialog 
					ConcurrentDialogTask.setTask(new Runnable() {
						
						@Override
						public void run() {
							// TODO Auto-generated method stub
							model.displayFC(true, new Executor<File>() {

								@Override
								public void execute(final File input) {
									try {
										ItsOdtIOManager.getIoManager(UnoRuntime.queryInterface(XComponentLoader.class, frame.getController().getModel())).exportDocument(input.toURI(), UnoRuntime.queryInterface(XTextDocument.class, frame.getController().getModel()));
										String path = input.toURI().getPath();
										if(!path.endsWith(".odt")) path += ".odt";
										try {
											ItsSerializerService.serialize(its, path + ".its-manager.ser");
										} catch (Exception e) {
											// TODO Auto-generated catch block
											e.printStackTrace();
										}
										
										// export to its
										ItsProcessor processor = new ItsProcessor();
										URI output = URI.create(input.toURI().getPath() + ".its");
										processor.forward(URI.create(path), output, its);
										
										
										XLIFFProcessor xliff = new XLIFFProcessor();
										xliff.forward(output, URI.create(output.getPath() + ".xlf"), LocaleId.fromString("de"), LocaleId.fromString("en"));
										
									} catch (Exception e) {
										e.printStackTrace();
										
										// reset the lock
										ConcurrentDialogTask.reset();
									} 
								}
							}); // end model launch save dialog
						}
					}).start();
					
				}
				
				if( aURL.Path.compareTo(URL_IMPORT) == 0) {
					
					// suppress the dual display of the open dialog
					ConcurrentDialogTask.setTask(new Runnable() {
						
						@Override
						public void run() {
							// TODO Auto-generated method stub
							model.displayFC(false, new Executor<File>() {

								@Override
								public void execute(final File input) {
									// TODO: import Files (XLIFF or ODT)
									
									XLIFFProcessor x2o		= new XLIFFProcessor();
									ItsProcessor itso2o		= new ItsProcessor();
									try {
										URI uri = input.toURI();
										if(uri.getPath().endsWith(".xlf")) {
											
											
											// transform xliff document to itstree
											ItsTree<Element> odttree = x2o.backward(uri, null, null, null);
											
											// obtain text document
											XTextDocument xTextDocument = UnoRuntime.queryInterface( XTextDocument.class, frame.getController().getModel());
											
											// create odt stub
											Object desktop = context.getServiceManager().createInstanceWithContext( "com.sun.star.frame.Desktop", context);
											XDesktop xDesktop = UnoRuntime.queryInterface(XDesktop.class, desktop);
											XComponentLoader loader = UnoRuntime.queryInterface(XComponentLoader.class, xDesktop);
											
											// create target document to load by Openoffice
											URI target = File.createTempFile("imported_xliff", ".odt").toURI();
											log("created target", target);
											
											// create temp file to overwrite content.xml
											URI odtstub = File.createTempFile("odtstub", ".odt").toURI();
											log("created odtstub", odtstub);
											
											// save current document to overwrite content.xml
											ItsOdtIOManager.getIoManager(loader).exportDocument(odtstub, xTextDocument);
											
											// copy file, because of file locking
											odtstub = Util.mkTmpCopy(odtstub, ".odt");
											log("copied odtstub", odtstub);
											
											// overwrite content.xml
											IOResult result = itso2o.backward(odttree, odtstub, target);
											log("written result?", result);
											
											// load imported document
											try {
												ItsOdtIOManager.getIoManager(loader).importDocument(result.getResourcePath().getPath());
											} catch (Exception e) {
												// TODO Auto-generated catch block
												e.printStackTrace();
											}
											log("successfully imported?", "true");
											
											// update time
											lastTime = System.currentTimeMillis();
										} 
									} catch(final Exception e) {
										e.printStackTrace();
										
										// reset the lock
										ConcurrentDialogTask.reset();
									}
									finally {
										ItsSessionManager.clearSession(frame.getController().getModel().getURL(), frame);
									}
									
									
								}
							}); // end model display import
						}
					}).start();
					
					
				}
				
			}
		}
		catch (java.lang.Exception ex) {
			log(" dispatch() exception " + Arrays.toString(ex.getStackTrace()) );
		}
	}

	@Override
	public void removeStatusListener(XStatusListener arg0, URL arg1) {
	}

	@Override
	public void initialize(Object[] arg0) throws com.sun.star.uno.Exception {
		frame = UnoRuntime.queryInterface(XFrame.class, arg0[0]);
	}

	@Override
	public XDispatch queryDispatch(final URL aURL, String arg1, int arg2) {
		if(aURL.Protocol.compareTo("org.openoffice.demo.ilomain:") == 0)
		{
			if(Util.matchSingle(new MatcherOp<String>() {
				@Override
				public boolean match(String input) {
					return aURL.Path.compareTo(input) == 0;
				}
			}, 
					Arrays.asList(
							URL_LOCALEFILTER, 
							URL_LOCALIZATION, 
							URL_TERMINOLOGY,
							URL_TRANSLATE,
							URL_EXPORT,
							URL_IMPORT))) {
				return this;
			}
		}
		return null;
	}

	@Override
	public XDispatch[] queryDispatches(DispatchDescriptor[] arg0) {
		return null;
	}

	@Override
	public String getImplementationName() {
		return null;
	}

	@Override
	public String[] getSupportedServiceNames() {
		return null;
	}

	@Override
	public boolean supportsService(String arg0) {
		return false;
	}

	@Override
	public void controlEvent(ControlEvent event) {
	}


	
} // end of class

