 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package org.openoffice.demo;

/**
 * The Range class is to be used in conjunction with bookmarks.
 * It provides the means to detect selections and provides basic
 * set operations
 * 
 * @author mkandora
 *
 */
public class Range {

	private String name;
	private int start;
	private int end;

	/**
	 * Creates a range with a name and a start and end index
	 * @param name the name of this range
	 * @param start the start of this range
	 * @param end the end of this range
	 */
	public Range(String name, int start, int end) {
		this.name 	= name;
		this.start 	= start;
		this.end 	= end;
	}
	
	/**
	 * Creates a Range without an identifier
	 * 
	 * @param start the start index
	 * @param end the end index
	 */
	public Range(final int start, final int end) {
		this("", start, end);
	}

	/**
	 * gets an optional identifier. used for ids
	 * @return
	 */
	public String getName() {
		return name;
	}

	/**
	 * gets the start of range
	 * @return
	 */
	public int getStart() {
		return start;
	}
	
	/**
	 * gets the end of range
	 * @return
	 */
	public int getEnd() {
		return end;
	}

	/**
	 * Returns true if this range contains another range
	 * 
	 * @param range
	 * @return
	 */
	public boolean contains(final Range range) {
		return range != null && (range.getStart() >= getStart() && range.getEnd() <= getEnd());
	}
	
	/**
	 * Returns true, if this range exactly matches the provided range
	 * @param range
	 * @return true, if exact match
	 */
	public boolean match(final Range range) {
		return range != null && (range.getStart() == getStart() && range.getEnd() == getEnd());
	}
	
	
	@Override
	public String toString() {
		return "Range [name=" + name + ", start=" + start + ", end=" + end
				+ "]";
	}

}
