 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG f�r digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.api.ds;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import de.init.its.ds.ItsAttributes;

/*
 * ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================
 * Defines an its tag element as a tree structure.
 * 
 * @author m.kandora
 *
 */
public interface IitsTagElement extends Serializable {
	
	/**
	 * Adds an IITagsElement to this element
	 * @param element
	 */
	public void addChild(final IitsTagElement element);
	
	
	/**
	 * Removes a child from this branch. No further operation is done here
	 * eg. all children will NOT be moved one level up. 
	 * 
	 * @param child
	 */
	public void removeChild(final IitsTagElement child);
	
	/**
	 * Returns all child its tag elements, if any
	 * @return
	 */
	public List<IitsTagElement> getChildren();
	
	/**
	 * Returns if this Element has any child nodes
	 * @return
	 */
	public boolean hasChildren();
	
	/**
	 * Returns true, if this element is an orphan (has no parent nor children nor any values)
	 * 
	 * @return
	 */
	public boolean isEmpty();
	
	/**
	 * Returns the id of this element
	 * 
	 * @return
	 */
	public IitsId getId();
	
	/**
	 * Sets the id of this its element
	 * @param id
	 */
	public void setId(final IitsId id);
	
	/**
	 * returns the next {@linkplain IitsTagElement}
	 * @return
	 */
	public IitsTagElement getNextChild();
	
	/**
	 * returns true if there are more child elements available
	 * @return
	 */
	public boolean hasMoreChildren();
	
	/**
	 * returns the parent element
	 * @return
	 */
	public IitsTagElement getParentElement();
	
	/**
	 * Sets the ItsTageElements parent
	 * @param e
	 */
	public void setParentElement(final IitsTagElement e);
	
	/**
	 * returns true if this element is parent
	 * @return
	 */
	public boolean isParent();
	
	/**
	 * Returns true if this element has a parent element
	 * 
	 * @return true, if element has parent
	 */
	public boolean hasParent();
	
	/**
	 * Returns true if this tag element is the root element
	 * (eg. is parent and has no parent element)
	 *  
	 * @return true if element is root element
	 */
	public boolean isRoot();
	
	/**
	 * set this element as parent
	 * 
	 * @param parent
	 */
	public void setParent(final boolean parent);
	
	/**
	 * Returns a list of all its attributes
	 * @return
	 */
	public List<IitsAttribute> getAttributes();
	
	/**
	 * Convenience method to render all attributes as map
	 * @return
	 */
	public Map<ItsAttributes, List<String>> renderAttributes();
	
	/**
	 * adds an its attribute to the element
	 * 
	 * @param itsAttribute
	 */
	public void addAttribute(final IitsAttribute itsAttribute);
	
	/**
	 * adds an its attribute to the element
	 * 
	 * @param dcname the name of the data category attribute
	 * @param dcvalue the value of the data category attribute
	 */
	public void addAttribute(final String dcname, final String dcvalue);
	
	/**
	 * adds an its attribute to the element
	 * 
	 * @param dcname the name of the data category attribute
	 * @param dcvalue the value of the data category attribute
	 */
	public void addAttribute(final ItsAttributes dcname, final String dcvalue);
	
	/**
	 * Adds all attributes to this ITS tag Element 
	 *  
	 * @param attributes a collection of {@linkplain IitsAttribute}
	 */
	public void addAllAttributes(final Collection<IitsAttribute> attributes);
	
	/**
	 * returns the last inserted child from this element
	 * @return {@link IitsTagElement}
	 */
	public IitsTagElement getLastChild();
	
	/**
	 * Updates this elements values 
	 * 
	 * @param values
	 */
	public void update(final Map<ItsAttributes, List<String>> values);
	
	/**
	 * resets the iterator
	 */
	public void reset();
}
