 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.api.process;

import de.init.its.spi.ItsElementManager;

/**
 * 
 * Global Its ODT Document Parser and Writer Configuration
 * 
 * @author mkandora
 *
 */
public final class IitsParserConfig {
	
	private IitsParserConfig() {}
	
	/**
	 * Its namespace uri. 
	 * <i><b>NOTICE:</b> the namespace will be updated in near future, once its is a recommendation</i>
	 */
	public static final String ITS_NAMESPACE_URI = "http://www.w3.org/2005/11/its";
	
	/**
	 * The content.xml of an ODT File
	 */
	public static final String ODT_CONTENT_XML	= "content.xml";

	public static final String ODT_ELEMENT_BODY = "office:body";
	
// ------------------------ styling block
	/**
	 * Declares the ODT Style Configuration block
	 */
	public static final String ODT_STYLE_CONFIGURATION_ELEMENT = "office:automatic-styles";
	
	/**
	 * Declares the ODT Style Configuration Element
	 */
	public static final String ODT_STYLE_ELEMENT = "style:style";
	
	/**
	 * Declares the style name attribute on {@linkplain #ODT_STYLE_ELEMENT}
	 */
	public static final String ODT_STYLE_ELEMENT_ATTRIBUTE_NAME = "style:name";
	
	/**
	 * Declares the attribute name used by ODT to reference styles
	 */
	public static final String ODT_STYLE_ELEMENT_STYLE_ATTRIBUTE_NAME = "text:style-name";
	
	/**
	 * Declares the style family attribute name
	 */
	public static final String ODT_STYLE_ELEMENT_ATTRIBUTE_STYLE_FAMILY = "style:family";
	
	/**
	 * Declares the default style family name
	 */
	public static final String ODT_DEFAULT_STYLE_FAMILY = "text";
	
	/**
	 * Declares the default prefix for new styling ids
	 */
	public static final String ODT_DEFAULT_TEXT_STYLE_PREFIX = "T";
	
	/**
	 * Declares a text style property
	 */
	public static final String ODT_STYLE_TEXT_PROPERTY_ELEMENT = "style:text-properties";
	
	
// ------------------------ end styling block	
	
	/**  
	 * 	The XML namesspace prefix for document element nodes. Seems bogus, has been proven
	 * applicable in parsing :)
	 */
	public static final String XML_NAMESPACE_PREFIX = "xmlns";
	
	/**
	 * The ODT File Bookmark begin element name with namespace
	 */
	public static final String ODT_FILE_BOOKMARK_BEGIN_ELEMENT_NAME_NS = "text:bookmark-start";
	
	/**
	 * The ODT File Bookmark end element name with namespace
	 */
	public static final String ODT_FILE_BOOKMARK_END_ELEMENT_NAME_NS 	= "text:bookmark-end";
	
	/**
	 * The ODT File Bookmark start element name
	 */
	public static final String ODT_FILE_BOOKMARK_BEGIN_ELEMENT_NAME 	= "bookmark-start";
	
	/**
	 * The ODT File Bookmark end element name 
	 */
	public static final String ODT_FILE_BOOKMARK_END_ELEMENT_NAME 		= "bookmark-end";
	
	/**
	 * The ODT File Bookmark namespace prefix
	 * 
	 * FIXME: rename to text element namespace prefix ?
	 */
	public static final String ODT_FILE_BOOKMARK_ELEMENT_NS_PREFIX 	= "text";
	
	/**
	 * The ITS Attribute Namespace prefix
	 */
	public static final String ITS_ELEMENT_NS_PREFIX = "its";
	
	/**
	 * The internally used text:book-mark element attribute to resolve 
	 * its data categories managed by id via {@linkplain ItsElementManager}
	 */
	public static final String ODT_FILE_BOOKMARK_REFERENCING_ATTRIBUTE_NAME = "text:name";
	
	/**
	 * Target ODT XML element for use with ITS Datacategory.
	 * This element will be used to add ITS Datacategories
	 */
	public static final String ODT_FILE_TARGET_ITS_ELEMENT = "span";
	
	public static final String ODT_FILE_TARGET_ITS_ELEMENT_WITH_PREFIX = "text:span";
	
	
	/**
	 * This flag will be used to indicate the processing of unprocessed ODT Document
	 * Files. eg. no bookmarks have been replaced.
	 */
	public static final Integer READ_UNPROCESSED_ODT_FILE = 0x1;

	/**
	 * Default XML declaration
	 */
	public static final Object XML_DECLARATION = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>"; 
	
	/**
	 * Convenience method to return an element with a proper namespace
	 * for Bookmark processing
	 * 
	 * @param attribute the attribute name
	 * @return the attribute with namespace constructed in front
	 */
	public static String getNameSpacedAttributed(final String attribute) {
		return ODT_FILE_BOOKMARK_ELEMENT_NS_PREFIX.concat(":" + attribute);
	}
}
