 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.api.spi;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import de.init.its.api.ds.IitsAttribute;
import de.init.its.ds.ItsAttributes;
import de.init.its.exception.UnsupportedItsAttributeException;

/**
 * Support class for reading the configuration file, that defines
 * a list of supported ITS markup
 * 
 * @author mkandora
 *
 */
public interface IitsAttributeManager extends Serializable {
	
	/**
	 * returns the respective {@linkplain IitsAttribute} as a copy
	 * 
	 * @param itsAttributeName the plain name of the its attribute
	 * @return an {@linkplain IitsAttribute}
	 * @throws UnsupportedItsAttributeException
	 */
	public IitsAttribute getAttribute(final String itsAttributeName) throws UnsupportedItsAttributeException;
	
	/**
	 * returns the respective {@linkplain IitsAttribute} from predefined 
	 * {@linkplain ItsAttributes} as a copy
	 * 
	 * @param attribute the {@linkplain ItsAttributes} enum
	 * @return an {@linkplain IitsAttribute}
	 * @throws UnsupportedItsAttributeException
	 */
	public IitsAttribute getAttribute(final ItsAttributes attribute) throws UnsupportedItsAttributeException;
	
	/**
	 * Returns all available its attribute as provided by the configuration
	 * 
	 * @return a collection with {@linkplain IitsAttribute}
	 */
	public Collection<IitsAttribute> getAllAvailableAttributes();
	
	/**
	 * Validates a new itsAttribute for correctness
	 * 
	 * @return
	 */
	public ItsError validate(final String itsAttributeName, final String itsAttributeValue);
}
