 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.ds;

import de.init.its.api.ds.IitsAttribute;

/**
 * 
 * Its Attribute Data Container Class
 * 
 * @author m.kandora
 *
 */
public class ItsAttribute implements IitsAttribute {
	
	private String itsAttributeName;
	private String itsAttributeValue;
	
	public ItsAttribute() {
		this("","");
	}
	
	public ItsAttribute(final String itsAttributeName, final String itsAttributeValue) {
		this.itsAttributeName 	= itsAttributeName;
		this.itsAttributeValue 	= itsAttributeValue;
	}
	
	@Override
	public String getItsAttributeName() {
		return itsAttributeName;
	}
		
	@Override
	public String getItsAttributeValue() {
		return itsAttributeValue;
	}

	@Override
	public IitsAttribute getDefaults() {
		return null;
	}

	@Override
	public void setAttribute(String name, String value) {
		this.itsAttributeName = name;
		this.itsAttributeValue = value;
	}

	@Override
	public void copy(IitsAttribute attribute) {
		this.itsAttributeName 	= attribute.getItsAttributeName();
		this.itsAttributeValue 	= attribute.getItsAttributeValue();
	}

	@Override
	public String toString() {
		return "ItsAttribute [itsAttributeName=" + itsAttributeName
				+ ", itsAttributeValue=" + itsAttributeValue + "]";
	}
	
	@Override
	public IitsAttribute copy() {
		IitsAttribute clone = new ItsAttribute();
		clone.setAttribute(itsAttributeName, itsAttributeValue);
		return clone;
	}

	@Override
	public void setAttributeName(String name) {
		this.itsAttributeName = name;
	}

	@Override
	public void setAttributeValue(String value) {
		this.itsAttributeValue = value;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((itsAttributeName == null) ? 0 : itsAttributeName.hashCode());
		result = prime
				* result
				+ ((itsAttributeValue == null) ? 0 : itsAttributeValue
						.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ItsAttribute other = (ItsAttribute) obj;
		if (itsAttributeName == null) {
			if (other.itsAttributeName != null)
				return false;
		} else if (!itsAttributeName.equals(other.itsAttributeName))
			return false;
		if (itsAttributeValue == null) {
			if (other.itsAttributeValue != null)
				return false;
		} else if (!itsAttributeValue.equals(other.itsAttributeValue))
			return false;
		return true;
	}
	
	
}
