 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.ds;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * The basic set for all four implemented DataCategories of ITS 2.0
 * 
 * @author mkandora
 *
 */
public enum ItsAttributes 
{
	translate(DataCategories.TRANSLATE, "translate", "(yes|no)", "yes"), 
	locNote(DataCategories.LOCALIZATIONNOTE, "locNote", ".*", ""), 
	locNoteRef(DataCategories.LOCALIZATIONNOTE, "locNoteRef", ".*", ""), 
	locNoteType(DataCategories.LOCALIZATIONNOTE, "locNoteType", "(alert|describtion)", "description"), 
	term(DataCategories.TERMINOLOGY, "term","(yes|no)", "no"), 
	termInfoRef(DataCategories.TERMINOLOGY, "termInfoRef", ".*", ""), 
	termConfidence(DataCategories.TERMINOLOGY, "termConfidence", "(\\d{0,1}(\\.\\d*)?)", "1.0"),
	annotatorsRef(DataCategories.TERMINOLOGY, "annotatorsRef", "*", ""),
	localeFilterList(DataCategories.LOCALEFILTER, "localeFilterList", "(\\*\\,?\\s)?(([a-z]{2}((\\-[A-Z]{2}){0,2})?(\\,)?(\\s)?|\\*{1})*)", ""),
	localeFilterType(DataCategories.LOCALEFILTER, "localeFilterType", "(include|exclude)", "");
	
	
	/**
	 * Defines the four basic ITS DataCategory names, that are implemented
	 * by ]init[ AG 
	 * 
	 * @author mkandora
	 *
	 */
	public enum DataCategories {
		TRANSLATE(translate), 
		LOCALEFILTER(localeFilterList, localeFilterType), 
		LOCALIZATIONNOTE(locNote, locNoteRef, locNoteType), 
		TERMINOLOGY(term, termConfidence, termInfoRef);
		
		// the internally used set of itsAttributes according for a specific data category
		Set<ItsAttributes> set;
		
		private DataCategories(final ItsAttributes...attributes) {
			set = new HashSet<ItsAttributes>();
			set.addAll(Arrays.asList(attributes));
		}
		
		public Set<ItsAttributes> retrieveAttributes() {
			return set;
		}
	}
	
	private final DataCategories dataCategory;
	private final String attributeName;
	private final String pattern;
	private final String _default;
	
	
	
	private ItsAttributes(final DataCategories dataCategory, final String attributeName, final String pattern, final String _default) {
		this.dataCategory = dataCategory;
		this.attributeName = attributeName;
		this.pattern = pattern;
		this._default = _default;
	}

	
	public DataCategories getDataCategory() {
		return dataCategory;
	}
	
	public String getAttributeName() {
		return attributeName;
	}
	
	public String getDefaultValue() {
		return _default;
	}
	
	public boolean validate(final String input) {
		return input != null && input.matches(pattern);
	}
	
	public static void main(String[] args) {
		System.out.println(ItsAttributes.localeFilterList.validate("*, sq-AL"));
	}
	
}
