 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.ds;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import de.init.its.ds.ItsAttributes.DataCategories;

/**
 * Service class to obtain data category mapping for the name 
 * and the respective attributes 
 * 
 * @author mkandora
 *
 */
public class ItsDataCategoryResolver {
	
	private Map<String, DataCategories> mapping;
	private Map<DataCategories, Set<ItsAttributes>> rMapping;
	private final static ItsDataCategoryResolver resolver = new ItsDataCategoryResolver();
	
	/**
	 * Returns a singletone instance of the {@link ItsDataCategoryResolver}
	 * @return
	 */
	public static ItsDataCategoryResolver getResolver() {
		return resolver;
	}
	
	private ItsDataCategoryResolver() {
		this.mapping = new HashMap<String, ItsAttributes.DataCategories>();
		this.rMapping  = new HashMap<ItsAttributes.DataCategories, Set<ItsAttributes>>();
		put(ItsAttributes.translate.name(),DataCategories.TRANSLATE); 
		put(ItsAttributes.locNote.name(),DataCategories.LOCALIZATIONNOTE); 
		put(ItsAttributes.locNoteRef.name(),DataCategories.LOCALIZATIONNOTE); 
		put(ItsAttributes.locNoteType.name(),DataCategories.LOCALIZATIONNOTE); 
		put(ItsAttributes.term.name(),DataCategories.TERMINOLOGY); 
		put(ItsAttributes.termInfoRef.name(),DataCategories.TERMINOLOGY); 
		put(ItsAttributes.termConfidence.name(),DataCategories.TERMINOLOGY);
		put(ItsAttributes.annotatorsRef.name(),DataCategories.TERMINOLOGY);
		put(ItsAttributes.localeFilterList.name(),DataCategories.LOCALEFILTER);
		put(ItsAttributes.localeFilterType.name(),DataCategories.LOCALEFILTER);
	}
	
	/**
	 * Puts a bi-directional mapping for {@link DataCategories} and their respective attributes
	 * @param attributeName
	 * @param dc
	 */
	public void put(final String attributeName, final DataCategories dc) {
		this.mapping.put(attributeName, dc);
		
		if(rMapping.containsKey(dc)) {
			rMapping.get(dc).add(ItsAttributes.valueOf(attributeName));
		} else {
			Set<ItsAttributes> set = new HashSet<ItsAttributes>();
			set.add(ItsAttributes.valueOf(attributeName));
			rMapping.put(dc, set);
		}
	}
	
	/**
	 * Returns the {@link DataCategories} for an its attribute
	 * @param attributeName
	 * @return
	 */
	public DataCategories retrieveDataCategory(final String attributeName) {
		return this.mapping.get(attributeName);
	}
	
	/**
	 * Returns all atributes defined by a {@link DataCategories}
	 * @param dc
	 * @return
	 */
	public Set<ItsAttributes> retrieveAttributes(final DataCategories dc) {
		return rMapping.containsKey(dc) ? rMapping.get(dc) : new HashSet<ItsAttributes>();
	}
	
}
