 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.ds;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import de.init.its.api.ds.IitsAttribute;
import de.init.its.api.ds.IitsId;
import de.init.its.api.ds.IitsTagElement;

/**
 * This class represents an ITS node. Each node has a number of
 * its attributes. each attribute contains the local ITS attribute and
 * its value.
 * 
 * @author mkandora
 *
 */
public class ItsTagElement implements IitsTagElement {

	private IitsId id;
	private List<IitsTagElement> children;
	private List<IitsAttribute> itsAttributes;
	private boolean isParent;
	private IitsTagElement parent;
	private Iterator<IitsTagElement> iterator;
	private IitsTagElement lastchild;
	
	public ItsTagElement() {
		this.children 		= new ArrayList<IitsTagElement>();
		this.itsAttributes	= new ArrayList<IitsAttribute>();
		this.id 			= new ItsId();
		this.isParent		= true;
		this.parent			= null;
		
	}
	
	@Override
	public void addChild(final IitsTagElement element) {
		this.children.add(element);
		element.setParentElement(this);
		element.setParent(false);
		this.lastchild = element;
		this.setParent(true);
	}
	
	@Override
	public void removeChild(final IitsTagElement child) {
		
		// FIXME: is this enough for child removal?
		
		this.children.remove(child);
	}
	
	@Override
	public List<IitsTagElement> getChildren() {
		return children;
	}

	@Override
	public boolean hasChildren() {
		return !children.isEmpty();
	}

	@Override
	public IitsId getId() {
		return this.id;
	}
	
	@Override
	public void setId(final IitsId id) {
		this.id = id;		
	}

	@Override
	public IitsTagElement getNextChild() {
		if(this.iterator == null)
			this.iterator = children.iterator();
		return this.iterator.next();
	}
	
	@Override
	public boolean hasMoreChildren() {
		if(this.iterator == null)
			this.iterator = children.iterator();
		return this.iterator.hasNext();
	}

	@Override
	public IitsTagElement getParentElement() {
		return parent;
	}
	
	@Override
	public void setParentElement(final IitsTagElement e) {
		this.parent = e;
	}

	@Override
	public boolean isParent() {
		return isParent;
	}

	@Override
	public void setParent(final boolean parent) {
		this.isParent = parent;
	}

	@Override
	public List<IitsAttribute> getAttributes() {
		return itsAttributes;
	}
	
	@Override
	public Map<ItsAttributes, List<String>> renderAttributes() {
		Map<ItsAttributes, List<String>> map = new HashMap<ItsAttributes, List<String>>();
		for(IitsAttribute attrib : getAttributes()) {
			ItsAttributes pre = ItsAttributes.valueOf(attrib.getItsAttributeName());
			if(map.get(pre) == null) {
				List<String> values = new ArrayList<String>();
				values.add(attrib.getItsAttributeValue());
				map.put(pre, values);
				continue;
			}
			List<String> values = map.get(attrib);
			values.add(attrib.getItsAttributeValue());
			map.put(pre, values);
		}
		return map;
	}
	
	@Override
	public void addAttribute(final IitsAttribute itsAttribute) {
		synchronized (itsAttribute) {
			this.itsAttributes.add(itsAttribute);
		}
	}
	
	@Override
	public void addAttribute(String dcname, String dcvalue) {
		this.addAttribute(new ItsAttribute(dcname, dcvalue));
	}
	
	@Override
	public void addAttribute(ItsAttributes dcname, String dcvalue) {
		this.addAttribute(new ItsAttribute(dcname.getAttributeName(), dcvalue));
		
	}

	@Override
	public IitsTagElement getLastChild() {
		return lastchild;
	}

	@Override
	public void addAllAttributes(final Collection<IitsAttribute> attributes) {
		this.itsAttributes.addAll(attributes);
	}
	
	@Override
	public String toString() {
		return "ItsTagElement [id=" + id + ", itsAttributes=" + itsAttributes + "]";
	}

	@Override
	public boolean isRoot() {
		return isParent() && getParentElement() == null;
	}

	@Override
	public boolean hasParent() {
		return getParentElement() != null;
	}
	
	@Override
	public boolean isEmpty() {
		return !hasParent() && getAttributes().isEmpty() && getId().getString().isEmpty();
	}
	
	@Override
	public void reset() {
		this.iterator = null;
	}
	
	@Override
	public void update(final Map<ItsAttributes, List<String>> values) {
		
		Map<String, String> map = new HashMap<String, String>();
		for(IitsAttribute attr : getAttributes()) {
			map.put(attr.getItsAttributeName(), attr.getItsAttributeValue());
		}
		itsAttributes.clear();
		for(ItsAttributes its : values.keySet()) {
			String val = values.get(its).toString().replaceAll("(\\[|\\])", "").trim();
			map.put(its.name(), val);
			itsAttributes.add(new ItsAttribute(its.name(), val));
		}
	}
	
}
