 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.process;

import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.init.its.api.ds.IitsAttribute;
import de.init.its.api.ds.IitsTagElement;
import de.init.its.api.process.IitsParserConfig;
import de.init.its.api.process.OdtStyleConfiguration;
import de.init.its.ds.ItsAttributes;
import de.init.its.ds.ItsDataCategoryResolver;
import de.init.its.ds.ItsAttributes.DataCategories;

/**
 * Service class to provide the style section node for 
 * an ODT file and the appropriate style mapping
 * 
 * @author mkandora
 *
 */
public class ItsStyleProcessor {
	
	private Map<DataCategories, String> styleIdMapping;
	private int nextId;
	
	private static ItsStyleProcessor proc = new ItsStyleProcessor();
	private ItsDataCategoryResolver resolver;
	
	/**
	 * Returns an instance of the ItsStyleProcessor using an optional initializing 
	 * 
	 * @param init boolean flag to indicate initializing (reseting ids, reload style configuration)
	 * @return the instance
	 */
	public static ItsStyleProcessor getProcessor(boolean init) {
		return !init ? proc : (proc = new ItsStyleProcessor());
	}
	
	/**
	 * Returns a singleton instance of the styleprocessor uninitialized
	 * @return
	 */
	public static ItsStyleProcessor getProcessor() {
		return getProcessor(false);
	}
	
	private ItsStyleProcessor() {
		this.styleIdMapping = new HashMap<DataCategories, String>();
		this.styleIdMapping.put(DataCategories.TRANSLATE, "T1");
		this.styleIdMapping.put(DataCategories.LOCALIZATIONNOTE, "T2");
		this.styleIdMapping.put(DataCategories.LOCALEFILTER, "T3");
		this.styleIdMapping.put(DataCategories.TERMINOLOGY, "T4");
		
		this.resolver = ItsDataCategoryResolver.getResolver();
	}
	
	/**
	 * Returns the style id 
	 * 
	 * @param dc the data category to look for a style
	 * @return the style id as string
	 */
	public String getStyleId(final DataCategories dc) {
		return this.styleIdMapping.get(dc);
	}
	
	/**
	 * Returns the style id for an {@link IitsTagElement}
	 * 
	 * @param element
	 * @return
	 */
	public String getStyleId(final IitsTagElement element) {
		// the implementation only allows to have one dc category 
		// per IitsTagElement, hence the first match will provide the DC
		// and we can safely return a style id
		for(IitsAttribute ia : element.getAttributes()) {
			return getStyleId(resolver.retrieveDataCategory(ia.getItsAttributeName()));
		}
		return "T1";
	}
	
}