 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.process.re;

import static de.init.its.process.re.XLIFFReader.XLIFF2ITSMapperService.convert;
import static de.init.its.process.re.XLIFFReader.XLIFF2ITSMapperService.detect;

import java.io.File;
import java.net.URI;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import net.sf.okapi.common.Event;
import net.sf.okapi.common.LocaleId;
import net.sf.okapi.common.filterwriter.ext.XLIFFWriter;
import net.sf.okapi.common.resource.RawDocument;
import de.init.its.ds.ItsAttributes;
import de.init.its.ds.ItsAttributes.DataCategories;
import de.init.its.okapi.ext.IFilter;
import de.init.its.okapi.ext.ODFFilter;
import de.init.its.process.IOResult;
import de.init.its.process.support.Element;
import de.init.its.process.support.ItsTree;
import de.init.its.util.Util;
//import net.sf.okapi.common.filters.IFilter;
/**
 * Service class to write and read XLIFF Files from and to ITS ODT files This
 * class makes heavy use of the filtering mechanisms of the OKAPI Framework to
 * access all TEXT_UNITS. 
 * 
 * @author m.kandora
 * 
 */
public class XLIFFProcessor {
	
	/**
	 * Internally stored ODT Stub
	 */
//	private final static String ODTSTUB = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><office:document-content	xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\" xmlns:its=\"http://www.w3.org/2005/11/its\" xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\"	xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\"	xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\"	xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\"	xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\"	xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\"	xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\"	xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\"	xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\"	xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\"	xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\"	xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\"	xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\"	xmlns:ooo=\"http://openoffice.org/2004/office\" xmlns:ooow=\"http://openoffice.org/2004/writer\"	xmlns:oooc=\"http://openoffice.org/2004/calc\" xmlns:dom=\"http://www.w3.org/2001/xml-events\"	xmlns:xforms=\"http://www.w3.org/2002/xforms\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"	xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:rpt=\"http://openoffice.org/2005/report\"	xmlns:of=\"urn:oasis:names:tc:opendocument:xmlns:of:1.2\" xmlns:xhtml=\"http://www.w3.org/1999/xhtml\"	xmlns:grddl=\"http://www.w3.org/2003/g/data-view#\" xmlns:officeooo=\"http://openoffice.org/2009/office\"	xmlns:tableooo=\"http://openoffice.org/2009/table\" xmlns:drawooo=\"http://openoffice.org/2010/draw\"	xmlns:calcext=\"urn:org:documentfoundation:names:experimental:calc:xmlns:calcext:1.0\"	xmlns:field=\"urn:openoffice:names:experimental:ooo-ms-interop:xmlns:field:1.0\"	xmlns:formx=\"urn:openoffice:names:experimental:ooxml-odf-interop:xmlns:form:1.0\"	xmlns:css3t=\"http://www.w3.org/TR/css3-text/\" office:version=\"1.2\">	<office:scripts />	<office:font-face-decls>		<style:font-face style:name=\"Mangal1\" svg:font-family=\"Mangal\" />		<style:font-face style:name=\"Times New Roman\"			svg:font-family=\"&apos;Times New Roman&apos;\"			style:font-family-generic=\"roman\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"Arial\" svg:font-family=\"Arial\"			style:font-family-generic=\"swiss\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"Mangal\" svg:font-family=\"Mangal\"			style:font-family-generic=\"system\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"Microsoft YaHei\"			svg:font-family=\"&apos;Microsoft YaHei&apos;\"			style:font-family-generic=\"system\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"SimSun\" svg:font-family=\"SimSun\"			style:font-family-generic=\"system\" style:font-pitch=\"variable\" />	</office:font-face-decls>	<office:automatic-styles>		<!-- translate -->		<style:style style:name=\"T1\" style:family=\"text\">			<style:text-properties fo:background-color=\"#aaffaa\" />		</style:style>		<!-- localization note -->		<style:style style:name=\"T2\" style:family=\"text\">			<style:text-properties				style:text-underline-style=\"solid\" style:text-underline-width=\"bold\"				style:text-underline-color=\"font-color\" fo:background-color=\"#bad666\"				style:text-overline-style=\"solid\" style:text-overline-width=\"bold\"				style:text-overline-color=\"font-color\" />		</style:style>		<!-- locale filter -->		<style:style style:name=\"T3\" style:family=\"text\">			<style:text-properties				style:text-underline-style=\"solid\" style:text-underline-width=\"bold\"				style:text-underline-color=\"font-color\" fo:background-color=\"#00ff00\"				style:text-overline-style=\"solid\" style:text-overline-width=\"bold\"				style:text-overline-color=\"font-color\" />		</style:style>		<!-- terminology -->		<style:style style:name=\"T4\" style:family=\"text\">			<style:text-properties				style:text-underline-style=\"solid\" style:text-underline-width=\"bold\"				style:text-underline-color=\"font-color\" fo:background-color=\"#ff0000\"				style:text-overline-style=\"solid\" style:text-overline-width=\"bold\"				style:text-overline-color=\"font-color\" />		</style:style>	</office:automatic-styles>	<office:body>		<office:text>		<!-- content comes here -->		</office:text>	</office:body></office:document-content>";
	private final static String ODTSTUB = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><office:document-content	xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\"	xmlns:its=\"http://www.w3.org/2005/11/its\" xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\"	xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\"	xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\"	xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\"	xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\"	xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\"	xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\"	xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\"	xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\"	xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\"	xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\"	xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\"	xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\"	xmlns:ooo=\"http://openoffice.org/2004/office\" xmlns:ooow=\"http://openoffice.org/2004/writer\"	xmlns:oooc=\"http://openoffice.org/2004/calc\" xmlns:dom=\"http://www.w3.org/2001/xml-events\"	xmlns:xforms=\"http://www.w3.org/2002/xforms\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"	xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:rpt=\"http://openoffice.org/2005/report\"	xmlns:of=\"urn:oasis:names:tc:opendocument:xmlns:of:1.2\" xmlns:xhtml=\"http://www.w3.org/1999/xhtml\"	xmlns:grddl=\"http://www.w3.org/2003/g/data-view#\" xmlns:officeooo=\"http://openoffice.org/2009/office\"	xmlns:tableooo=\"http://openoffice.org/2009/table\" xmlns:drawooo=\"http://openoffice.org/2010/draw\"	xmlns:calcext=\"urn:org:documentfoundation:names:experimental:calc:xmlns:calcext:1.0\"	xmlns:field=\"urn:openoffice:names:experimental:ooo-ms-interop:xmlns:field:1.0\"	xmlns:formx=\"urn:openoffice:names:experimental:ooxml-odf-interop:xmlns:form:1.0\"	xmlns:css3t=\"http://www.w3.org/TR/css3-text/\" office:version=\"1.2\">	<office:scripts />	<office:font-face-decls>		<style:font-face style:name=\"Mangal1\" svg:font-family=\"Mangal\" />		<style:font-face style:name=\"Times New Roman\"			svg:font-family=\"&apos;Times New Roman&apos;\"			style:font-family-generic=\"roman\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"Arial\" svg:font-family=\"Arial\"			style:font-family-generic=\"swiss\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"Mangal\" svg:font-family=\"Mangal\"			style:font-family-generic=\"system\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"Microsoft YaHei\"			svg:font-family=\"&apos;Microsoft YaHei&apos;\"			style:font-family-generic=\"system\" style:font-pitch=\"variable\" />		<style:font-face style:name=\"SimSun\" svg:font-family=\"SimSun\"			style:font-family-generic=\"system\" style:font-pitch=\"variable\" />	</office:font-face-decls>	<office:automatic-styles>		<!-- translate -->		<style:style style:name=\"T1\" style:family=\"text\">			<style:text-properties fo:background-color=\"#aaffaa\" />		</style:style>				<!-- localization note -->		<style:style style:name=\"T2\" style:family=\"text\">			<style:text-properties				style:text-underline-style=\"solid\" style:text-underline-width=\"bold\"				style:text-underline-color=\"font-color\" />		</style:style>				<!-- locale filter -->		<style:style style:name=\"T3\" style:family=\"text\">			<style:text-properties				style:text-overline-style=\"solid\" style:text-overline-width=\"bold\"				style:text-overline-color=\"font-color\" />		</style:style>		<!-- terminology -->		<style:style style:name=\"T4\" style:family=\"text\">			<style:text-properties				fo:background-color=\"#ffaaaa\"				/>		</style:style>	</office:automatic-styles>	<office:body>		<office:text>		<!-- content comes here -->		</office:text>	</office:body></office:document-content>";
	
	/**
	 * This represents the generated odtTree structure to write out an odt document
	 */
	private ItsTree<Element> odttree;

	/**
	 * Implements the forward process to create an XLIFF file from an ITS ODT
	 * resource
	 * 
	 * @param resource
	 *            the resource its odt file
	 * @param target
	 *            the target file uri for writing the xliff file
	 * @param sourceLocale
	 *            the source locale. use the static accessor methods to create a
	 *            new one
	 * @param targetLocale
	 *            the target locale
	 * @return the result of the operation
	 */
	public IOResult forward(final URI resource, final URI target,
			final LocaleId sourceLocale, final LocaleId targetLocale) {
		IOResult result = null;
		try {
			IFilter filter = new ODFFilter();
			XLIFFWriter writer = new XLIFFWriter();
			
			// extract xml content from ITS ODT File
			URI contentXML = Util.extractZipEntry(resource.getPath(),
					"content.xml").toURI();
			writer.create(new File(target.getPath()).getAbsolutePath(), null,
					sourceLocale, targetLocale, null, "temp.odt", null);
			writer.writeStartFile(null, null, null);

			// create raw document for filter processing
			RawDocument document = new RawDocument(contentXML, "utf-8",
					sourceLocale, targetLocale);

			// open the document and start processing only the text-units
			filter.open(document);
			while (filter.hasNext()) {
				Event evt = filter.next();
				switch (evt.getEventType()) {
				case TEXT_UNIT:
					writer.writeTextUnit(evt.getTextUnit());
					break;
				default:
					// NOP
					break;
				}
			}
			writer.writeEndFile();
			writer.close();

		} catch (Exception e) {
			e.printStackTrace();
		}

		return result;
	}

	/**
	 * Implements the backward process to generate ITS ODT Files from xliff
	 * files without loss of content information.
	 * 
	 * the conversion process from xliff to odt will consider each translation-unit 
	 * as text:p element in odt (this runs along with OKAPI to simplify things) the
	 * only reasonable source for converting documents back to odt format will be 
	 * the trans-unit target element, since this will contain the translated  resource.
	 * 
	 * @param resource
	 *            the xliff resource
	 * @param target
	 *            the target to generate the odt file
	 * @return the result of the operation
	 */
	public ItsTree<Element> backward(final URI resource, final URI target,
			final LocaleId sourceLocale, final LocaleId targetLocale) {
		
		// load the xliff reader to obtain a tree structure
		XLIFFReader xmlreader 			= new XLIFFReader();
		
		// read xliff file as tree
		ItsTree<Element> xlifftree 		= xmlreader.read(resource);
		
		// read odt stub
		this.odttree					= new XLIFFReader().read(ODTSTUB);
		
		// find body element inside odt document
		final ItsTree<Element> odtBody	= odttree.findByName("office", "body").findByName("office", "text");
		
		// find xliff body to begin extraction of content
		ItsTree<Element> xliffbody 		= xlifftree.findByName(null, "body");
		
		// the current element
		ItsTree<Element> odtCurrent 	= odtBody;
		
		// iterate over translation units
		for(ItsTree<Element> tu : xliffbody.getChildren()) 
		{
			
			// determine restype
			Element enclosing = getEnclosingElement(tu);
			
			// reset to office-text element
			final ItsTree<Element> paragraph = odtBody.insert(enclosing);
			odtCurrent = paragraph;
			
			// find xliff 'target' element to obtain the translated source
			ItsTree<Element> tuTarget = findByName(tu, "target");//tu.findAllByName(null, "target").getFirst();
			
			// assumption: a <g></g> block may indicate ITS content, all other subtrees may be simple spans
			ItsTree<Element> node 			= tuTarget;
			
			// used for children of xliff body element
			Stack<ItsTree<Element>> children = new Stack<ItsTree<Element>>();
			Collections.reverse(node.getChildren());
			children.addAll(node.getChildren());
			
			// core transformation steps
			while(!children.isEmpty()) {
				
				ItsTree<Element> child 	= children.pop();
				Element head 			= child.getHead();
				if(head.isText()) {
					// check, if parent has data category
					boolean hasIts = false;
					DataCategories dc = null;
					ItsTree<Element> test = child;
					while((test = test.getParent()) != null) {
						if(hasDataCategory(test) ) {
							dc = detect(test);
							hasIts = true;
							break;
						}
					}
					DataCategories current = detect(odtCurrent);
					if(current != null && !detect(odtCurrent).equals(dc)) odtCurrent = odtCurrent.getParent(); 
					// MK changed from getParent() to paragraph
					else if(!hasIts) odtCurrent = odtCurrent.getParent();
					if(odtCurrent.equals(odtBody)) odtCurrent = paragraph;
					
					// insert text node, do not modify structure
					odtCurrent = odtCurrent.insert(new Element(child.getHead().getText())).getParent();
				}
				
				if(hasDataCategory(child)) {
					// check, if parent has data category
					boolean hasIts = false;
					ItsTree<Element> test = child;
					while((test = test.getParent()) != null) {
						if(hasDataCategory(test) ) {
							hasIts = true;
							break;
						}
					}
					// MK changed from getParent() to paragraph
					if(!hasIts) odtCurrent = odtCurrent.getParent();
					if(odtCurrent.equals(odtBody)) odtCurrent = paragraph;
					
					odtCurrent = odtCurrent.insert(mkSpanElement(convert(child, detect(child))));
				}
				if(child.hasChildren()) {
					Collections.reverse(child.getChildren());
					children.addAll(child.getChildren());
					continue;
				}
			}
		}
		return odttree;
	}

	/**
	 * Returns the enclosing element type as determined by xliff "restype"
	 * 
	 * @param tu the translation units element node
	 * @return a new element containing the enclosing type
	 */
	private Element getEnclosingElement(ItsTree<Element> tu) {
		String value = tu.getHead().getAttributes().get("restype");
		String[] splitted = value.split("\\:");
		if(splitted.length != 2) return new Element("text", "p");
		String ns = splitted[0].substring("x-".length());
		String el = splitted[1];
		return new Element(ns, el);
	}
	
	
	/**
	 * Returns an element by name
	 * 
	 * @param t
	 * @param name
	 * @return
	 */
	protected ItsTree<Element> findByName(final ItsTree<Element> t, final String name) {
		for(ItsTree<Element> child : t.getChildren()) {
			if(!child.getHead().isText() && child.getHead().getName().equals(name)) {
				return child;
			}
		}
		return null;
	}
	
	/**
	 * Returns true, if XLIFF group element contains no more groups
	 * and is considered a leaf
	 * 
	 * @param id the id to look for
	 * @param group the group element 
	 * @return true, if leaf 
	 */
	public boolean isLeafGroup(final String id, final ItsTree<Element> group) {
		for(ItsTree<Element> child : group.getChildren()) {
			if(isGroup(child)) {
				if(!child.getHead().getAttributes().get("id").equals(id)) {
					return false;
				}
			}
			else return isLeafGroup(id, child);
		}
		return true;
	}

	/**
	 * Returns true, if this is a generic group
	 * @param child
	 * @return true, if generic group
	 */
	private boolean isGroup(ItsTree<Element> child) {
		return !child.getHead().isText() && child.getHead().getName().equals("g");
	}
	
	/**
	 * returns true, if this subtree contains its to xliff mapping
	 * 
	 * @param child the child to check for its data category
	 * @return true, if subtree has data category
	 */
	public boolean hasDataCategory(final ItsTree<Element> child) {
		return detect(child) != null;
	}
	
	/**
	 * Creates a spanning Element
	 * @param mapping
	 * @return
	 */
	protected Element mkSpanElement(final Map<ItsAttributes, List<String>> mapping) {
		return new Element("its", "span", mapping);
	}
	
}
