 /* ===========================================================================
 *   Copyright (C) 2013 by ]init[ AG für digitale Kommunikation
 * -----------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this library; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  See also the full LGPL text here: http://www.gnu.org/copyleft/lesser.html
 * ===========================================================================*/
package de.init.its.util;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URI;

import javax.sql.rowset.serial.SerialException;

import org.yaml.snakeyaml.serializer.SerializerException;

import de.init.its.ds.ItsDocument;
import de.init.its.spi.ItsElementManager;

/**
 * Serializer Service Class
 * 
 * @author mkandora
 *
 */
public class ItsSerializerService {

	private ItsSerializerService() {}

	/**
	 * Serializes an its document to a given path
	 * 
	 * @param path the path to persist the serialized object
	 * @param document the (eg. {@linkplain ItsDocument}) to serialise
	 * @throws Exception of serialisation is not possible (some )
	 */
	public synchronized static <T> void serialize(final T document, final String path) throws Exception {
		FileOutputStream fos = new FileOutputStream(path);
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(document);
		oos.flush();
		oos.close();
		fos.flush();
		fos.close();
	}
	
	/**
	 * Deserializes a persistet object. 
	 * 
	 * @param path the path of the serialized object
	 * @param clazz the expected type
 	 * @return the deserialized object
	 * @throws Exception if something fails
	 */
	public synchronized static <T> T deserialise(final String path, Class<T> clazz) throws SerializerException, Exception {
		T deser = null;
		FileInputStream fis 	= new FileInputStream(new URI(path).getPath());
		ObjectInputStream ois 	= new ObjectInputStream(fis);
		deser = clazz.cast(ois.readObject());
		ois.close();
		fis.close();
		return deser;
	}
	
}
